#pragma once

// STD / STL
#include <optional>
#include <mutex>

// Memory
#include "../../core/wm/memory_definitions.h"

namespace armarx::armem::server::ltm
{
    /// @brief Interface functions for the longterm memory classes
    class LongtermMemoryBase
    {
    public:
        struct AppendResult
        {
            std::vector<MemoryID> addedCoreSegments;
            std::vector<MemoryID> addedProviderSegments;
            std::vector<MemoryID> addedEntities;

            std::vector<MemoryID> addedSnapshots;
            std::vector<MemoryID> replacedSnapshots;
            std::vector<MemoryID> ignoredSnapshots;
        };

        struct ReloadResult
        {

        };

        void append(const armem::wm::Memory&);

        virtual void reload() = 0;
        virtual void convert(armem::wm::Memory&) = 0;
        virtual void encodeAndStore() = 0;

        // pass through to internal memory
        void setName(const std::string& name);

        // get merged internal memory
        armem::wm::Memory getCacheAndLutNotConverted() const;

    protected:
        void checkUpdateLatestSnapshot(const armem::wm::EntitySnapshot& newSnapshot);

        bool containsCoreSegment(const MemoryID&) const;
        bool containsProviderSegment(const MemoryID&) const;
        bool containsEntity(const MemoryID&) const;
        bool containsSnapshot(const MemoryID&) const;

    protected:
        /// Internal memory for data consolidated from wm to ltm (cache)
        armem::wm::Memory cache;
        mutable std::recursive_mutex cache_mutex;

        /// Internal memory for indexes (lut)
        armem::wm::Memory lut;
        mutable std::recursive_mutex lut_mutex;

        /// A map from entityID to its latest snapshot stored. When adding a new snapshot we compare it to the last one stored.
        std::map<std::string, const armem::wm::EntitySnapshot*> latestSnapshots;

    };
}
