#pragma once

// STD / STL
#include <mutex>
#include <optional>

// Base Class
#include "../LongtermMemoryBase.h"

// Data
# include "ConnectionManager.h"

namespace armarx::armem::server::ltm::mongodb
{
    /// @brief A memory storing data in mongodb (needs 'armarx memory start' to start the mongod instance)
    class MemoryManager :
        public LongtermMemoryBase
    {
        using Base = LongtermMemoryBase;

    public:
        MemoryManager() = default;

        void reload() override;
        void convert(armem::wm::Memory&) override;
        void encodeAndStore() override;

    protected:


    private:
        PoolClientPtr checkConnection() const; // return nullptr if not possible

        void mongodbInsertForeignKey(mongocxx::collection&, const std::string& key);

        bool mongodbContainsForeignKey(mongocxx::collection&, const std::string& key) const;
        bool containsCoreSegment(mongocxx::collection&, const MemoryID&) const;
        bool containsProviderSegment(mongocxx::collection&, const MemoryID&) const;
        bool containsEntity(mongocxx::collection&, const MemoryID&) const;
        bool containsSnapshot(mongocxx::collection&, const MemoryID&) const;

    public:
        ConnectionManager::MongoDBSettings dbsettings;

    private:
        static const constexpr char* MONGO_FOREIGN_KEY = "foreign_key";
        static const constexpr char* MONGO_ID = "id";
        static const constexpr char* MONGO_TIMESTAMP = "timestamp";
        static const constexpr char* MONGO_INSTANCES = "instances";
    };
}
