#pragma once

#include <RobotAPI/libraries/armem/core/DataMode.h>
#include <RobotAPI/libraries/armem/core/wm/memory_definitions.h>
#include <RobotAPI/libraries/armem/server/wm/memory_definitions.h>
#include <RobotAPI/libraries/armem/server/query_proc/base.h>


namespace armarx::armem::server::query_proc::wm
{
    static const base::QueryTarget queryTarget = query::data::QueryTarget::WM;
}
namespace armarx::armem::server::query_proc::wm::detail
{

    class HasDataMode
    {
    public:

        HasDataMode(armem::DataMode dataMode);


    protected:

        armem::DataMode dataMode;

    };



    template <class SourceEntityT>
    class EntityQueryProcessor :
        public base::EntityQueryProcessorBase<queryTarget, SourceEntityT, armem::wm::Entity>,
        public HasDataMode
    {
    public:

        EntityQueryProcessor(DataMode dataMode = DataMode::WithData) :
            HasDataMode(dataMode)
        {}


    protected:

        void addResultSnapshot(armem::wm::Entity& result, const typename SourceEntityT::EntitySnapshotT& snapshot) const
        {
            bool withData = (dataMode == DataMode::WithData);
            if (withData)
            {
                result.addSnapshot(server::wm::EntitySnapshot{ snapshot });
            }
            else
            {
                server::wm::EntitySnapshot copy = snapshot;
                copy.forEachInstance([](server::wm::EntityInstance & i)
                {
                    i.data() = nullptr;
                    return true;
                });
                result.addSnapshot(std::move(copy));
            }
        }

    };
}


namespace armarx::armem::server::query_proc::wm
{

    using EntityQueryProcessor = detail::EntityQueryProcessor<armem::wm::Entity>;


    class ProviderSegmentQueryProcessor :
        public base::ProviderSegmentQueryProcessorBase<queryTarget, armem::wm::ProviderSegment, armem::wm::ProviderSegment, EntityQueryProcessor >,
        public detail::HasDataMode
    {
    public:

        ProviderSegmentQueryProcessor(DataMode dataMode = DataMode::WithData);

    };


    class CoreSegmentQueryProcessor :
        public base::CoreSegmentQueryProcessorBase <queryTarget, armem::wm::CoreSegment, armem::wm::CoreSegment, ProviderSegmentQueryProcessor >,
        public detail::HasDataMode
    {
        using Base = base::CoreSegmentQueryProcessorBase<queryTarget, armem::wm::CoreSegment, armem::wm::CoreSegment, ProviderSegmentQueryProcessor>;
        using CoreSegment = armem::wm::CoreSegment;
        using ProviderSegment = armem::wm::ProviderSegment;

    public:

        CoreSegmentQueryProcessor(DataMode dataMode = DataMode::WithData);

    };


    class MemoryQueryProcessor :
        public base::MemoryQueryProcessorBase<queryTarget, armem::wm::Memory, armem::wm::Memory, CoreSegmentQueryProcessor>,
        public detail::HasDataMode
    {
    public:

        MemoryQueryProcessor(DataMode dataMode = DataMode::WithData);

    };

}


namespace armarx::armem::server::query_proc::wm_server
{

    using EntityQueryProcessor = wm::detail::EntityQueryProcessor<server::wm::Entity>;


    class ProviderSegmentQueryProcessor :
        public base::ProviderSegmentQueryProcessorBase<wm::queryTarget, server::wm::ProviderSegment, armem::wm::ProviderSegment, EntityQueryProcessor >,
        public wm::detail::HasDataMode
    {
    public:

        ProviderSegmentQueryProcessor(DataMode dataMode = DataMode::WithData);

    };


    class CoreSegmentQueryProcessor :
        public base::CoreSegmentQueryProcessorBase <wm::queryTarget, server::wm::CoreSegment, armem::wm::CoreSegment, ProviderSegmentQueryProcessor >,
        public wm::detail::HasDataMode
    {
        using Base = base::CoreSegmentQueryProcessorBase<wm::queryTarget, server::wm::CoreSegment, armem::wm::CoreSegment, ProviderSegmentQueryProcessor>;
        using CoreSegment = server::wm::CoreSegment;
        using ProviderSegment = server::wm::ProviderSegment;

    public:

        CoreSegmentQueryProcessor(DataMode dataMode = DataMode::WithData);


        using Base::process;

        /// Locks the core segment, then delegates back to `CoreSegmentQueryProcessorBase`.
        void process(
            armem::wm::CoreSegment& result,
            const armem::query::data::CoreSegmentQuery& query,
            const CoreSegment& coreSegment) const override;

    };


    class MemoryQueryProcessor :
        public base::MemoryQueryProcessorBase<wm::queryTarget, server::wm::Memory, armem::wm::Memory, CoreSegmentQueryProcessor>,
        public wm::detail::HasDataMode
    {
    public:

        MemoryQueryProcessor(DataMode dataMode = DataMode::WithData);

    };

}
