#include "SceneSnapshot.h"

#include <SimoxUtility/json.h>

#include <RobotAPI/libraries/ArmarXObjects/ObjectID.h>
#include <RobotAPI/libraries/ArmarXObjects/ObjectFinder.h>

// #include <RobotAPI/libraries/armem/core/json_conversions.h>


namespace armarx::armem::obj
{

    ObjectID SceneSnapshot::Object::getClassID() const
    {
        return ObjectID(className);
    }


    ObjectID SceneSnapshot::Object::getClassID(ObjectFinder& finder) const
    {
        ObjectID id = getClassID();
        if (id.dataset().empty())
        {
            if (std::optional<ObjectInfo> info = finder.findObject(id.className()))
            {
                return info->id();
            }
        }
        return id;
    }


    ObjectID SceneSnapshot::Object::getObjectID() const
    {
        return getClassID().withInstanceName(instanceName);
    }


    ObjectID SceneSnapshot::Object::getObjectID(ObjectFinder& finder) const
    {
        return getClassID(finder).withInstanceName(instanceName);
    }

}


void armarx::armem::obj::to_json(nlohmann::json& j, const SceneSnapshot::Object& rhs)
{
    //     j["instanceID"] = rhs.instanceID;
    j["class"] = rhs.className;
    j["instanceName"] = rhs.instanceName;
    j["collection"] = rhs.collection;
    j["position"] = rhs.position;
    j["orientation"] = rhs.orientation;
    j["jointValues"] = rhs.jointValues;
}


void armarx::armem::obj::from_json(const nlohmann::json& j, SceneSnapshot::Object& rhs)
{
    //     j.at("instanceID").get_to(rhs.instanceID);
    j.at("class").get_to(rhs.className);
    if (j.count("instanceName"))
    {
        j["instanceName"].get_to(rhs.instanceName);
    }
    j.at("collection").get_to(rhs.collection);
    j.at("position").get_to(rhs.position);
    j.at("orientation").get_to(rhs.orientation);
    if (j.count("jointValues"))
    {
        j.at("jointValues").get_to(rhs.jointValues);
    }
}



void armarx::armem::obj::to_json(nlohmann::json& j, const SceneSnapshot& rhs)
{
    j["objects"] = rhs.objects;
}


void armarx::armem::obj::from_json(const nlohmann::json& j, SceneSnapshot& rhs)
{
    j.at("objects").get_to(rhs.objects);
}


