/*
* This file is part of ArmarX.
*
* ArmarX is free software; you can redistribute it and/or modify
* it under the terms of the GNU General Public License version 2 as
* published by the Free Software Foundation.
*
* ArmarX is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
* GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License
* along with this program. If not, see <http://www.gnu.org/licenses/>.
*
* @author     Fabian Peller (fabian dot peller at kit dot edu)
* @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
*             GNU General Public License
*/

#pragma once

// STD/STL
#include <memory>
#include <string>

// ArmarX
#include <RobotAPI/interface/aron.h>
#include <RobotAPI/libraries/aron/core/Exception.h>
#include <RobotAPI/libraries/aron/core/Config.h>


namespace armarx::aron::typeIO
{
    class WriterInterface;
    typedef std::shared_ptr<WriterInterface> WriterInterfacePtr;

    class WriterInterface
    {
    public:
        struct WriteStartObjectInput
        {
            std::string name;
            type::Maybe maybe;
        };
        virtual void writeStartObject(const WriteStartObjectInput&) = 0;
        virtual void writeEndObject() = 0;

        struct WriteStartListInput
        {
            type::Maybe maybe;
        };
        virtual void writeStartList(const WriteStartListInput&) = 0;
        virtual void writeEndList() = 0;

        struct WriteStartDictInput
        {
            type::Maybe maybe;
        };
        virtual void writeStartDict(const WriteStartDictInput&) = 0;
        virtual void writeEndDict() = 0;

        struct WriteStartTupleInput
        {
            type::Maybe maybe;
        };
        virtual void writeStartTuple(const WriteStartTupleInput&) = 0;
        virtual void writeEndTuple() = 0;

        struct WriteStartPairInput
        {
            type::Maybe maybe;
        };
        virtual void writeStartPair(const WriteStartPairInput&) = 0;
        virtual void writeEndPair() = 0;

        struct WriteEigenMatrixInput
        {
            unsigned int rows;
            unsigned int cols;
            std::string type;
            type::Maybe maybe;
        };
        virtual void writeEigenMatrix(const WriteEigenMatrixInput&) = 0;

        struct WriteEigenQuaternionInput
        {
            std::string type;
            type::Maybe maybe;
        };
        virtual void writeEigenQuaternion(const WriteEigenQuaternionInput&) = 0;

        struct WriteImageInput
        {
            std::string pixelType;
            type::Maybe maybe;
        };
        virtual void writeImage(const WriteImageInput&) = 0;

        struct WriteIVTCByteImageInput
        {
            type::Maybe maybe;
        };
        virtual void writeIVTCByteImage(const WriteIVTCByteImageInput&) = 0;

        struct WriteOpenCVMatInput
        {
            type::Maybe maybe;
        };
        virtual void writeOpenCVMat(const WriteOpenCVMatInput&) = 0;

        struct WritePCLPointCloudInput
        {
            std::string type;
            type::Maybe maybe;
        };
        virtual void writePCLPointCloud(const WritePCLPointCloudInput&) = 0;

        struct WritePositionInput
        {
            type::Maybe maybe;
        };
        virtual void writePosition(const WritePositionInput&) = 0;

        struct WriteOrientationInput
        {
            type::Maybe maybe;
        };
        virtual void writeOrientation(const WriteOrientationInput&) = 0;

        struct WritePoseInput
        {
            type::Maybe maybe;
        };
        virtual void writePose(const WritePoseInput&) = 0;

        struct WritePrimitiveInput
        {
            type::Maybe maybe;
        };
        virtual void writeInt(const WritePrimitiveInput&) = 0;
        virtual void writeLong(const WritePrimitiveInput&) = 0;
        virtual void writeFloat(const WritePrimitiveInput&) = 0;
        virtual void writeDouble(const WritePrimitiveInput&) = 0;
        virtual void writeString(const WritePrimitiveInput&) = 0;
        virtual void writeBool(const WritePrimitiveInput&) = 0;
        virtual void writeTime(const WritePrimitiveInput&) = 0;

        virtual void writeKey(const std::string&) = 0;
        virtual void writeExtends() = 0;
    };
}
