/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <memory>
#include <string>
#include <unordered_map>

// ArmarX
#include <RobotAPI/libraries/aron/core/Exception.h>
#include <RobotAPI/libraries/aron/core/Concepts.h>
#include <RobotAPI/libraries/aron/core/Descriptor.h>
#include <RobotAPI/libraries/aron/core/Resolver.h>
#include <RobotAPI/libraries/aron/core/Path.h>

namespace armarx::aron
{
    template <typename Descriptor, typename Aron> //requires isAron<Aron>&& isTypeDescriptor<Descriptor>
    class Navigator;

    template <typename Descriptor, typename Aron> //requires isAron<Aron>&& isTypeDescriptor<Descriptor>
    using NavigatorPtr = std::shared_ptr<Navigator<Descriptor, Aron>>;

    template <typename Descriptor, typename Aron> //requires isAron<Aron>&& isTypeDescriptor<Descriptor>
    class Navigator
    {
    public:

        // constructors
        Navigator() = default;
        Navigator(const Descriptor& descriptor, const Path& path) :
            descriptor(descriptor),
            path(path)
        {
        }
        virtual ~Navigator() = default;

        // public member functions
        Descriptor getDescriptor() const
        {
            return descriptor;
        }

        Path getPath() const
        {
            return path;
        }

        std::string pathToString() const
        {
            return path.toString();
        }

        // virtual definitions
        virtual typename Aron::PointerType toAronPtr() const = 0;
        virtual std::string getName() const = 0;

    protected:
        // members
        const Descriptor descriptor;
        const Path path;
    };
}
