/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "Bool.h"

namespace armarx::aron::datanavigator
{
    /* constructors */
    BoolNavigator::BoolNavigator(const data::AronBoolPtr& o, const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::eBool, path),
        detail::NavigatorBase<data::AronBool, BoolNavigator>(o)
    {
    }

    BoolNavigator::BoolNavigator(const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::eBool, path)
    {
    }

    BoolNavigator::BoolNavigator(const bool d, const Path& path) :
        BoolNavigator(data::AronBoolPtr(new data::AronBool(d)), path)
    {
    }

    /* operators */
    bool BoolNavigator::operator==(const BoolNavigator& other) const
    {
        const auto& otherAron = other.toAronBoolPtr();
        if (aron->value != otherAron->value)
        {
            return false;
        }
        return true;
    }
    bool BoolNavigator::operator==(const BoolNavigatorPtr& other) const
    {
        if (!other)
        {
            return false;
        }
        return *this == *other;
    }


    /* static methods */
    BoolNavigatorPtr BoolNavigator::FromAronBoolPtr(const data::AronBoolPtr& aron)
    {
        return std::make_shared<BoolNavigator>(aron);
    }

    data::AronBoolPtr BoolNavigator::ToAronBoolPtr(const BoolNavigatorPtr& navigator)
    {
        return navigator ? navigator->toAronBoolPtr() : nullptr;
    }

    /* public member functions */
    data::AronBoolPtr BoolNavigator::toAronBoolPtr() const
    {
        return aron;
    }

    /* virtual implementations */
    std::string BoolNavigator::getName() const
    {
        return "data::AronBool";
    }

    bool BoolNavigator::fullfillsType(const typenavigator::NavigatorPtr& type) const
    {
        return type->getDescriptor() == type::Descriptor::eBool;
    }

    typenavigator::NavigatorPtr BoolNavigator::recalculateType() const
    {
        return std::make_shared<typenavigator::BoolNavigator>(getPath());
    }
}
