/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "Double.h"

// ArmarX
#include "Float.h"

namespace armarx::aron::datanavigator
{
    /* constructors */
    DoubleNavigator::DoubleNavigator(const data::AronDoublePtr& o, const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::eDouble, path),
        detail::NavigatorBase<data::AronDouble, DoubleNavigator>(o)
    {
    }

    DoubleNavigator::DoubleNavigator(const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::eDouble, path)
    {
    }

    DoubleNavigator::DoubleNavigator(const double d, const Path& path) :
        DoubleNavigator(data::AronDoublePtr(new data::AronDouble(d)), path)
    {
    }

    /* operators */
    bool DoubleNavigator::operator==(const DoubleNavigator& other) const
    {
        const auto& otherAron = other.toAronDoublePtr();
        if (this->aron->value != otherAron->value)
        {
            return false;
        }
        return true;
    }
    bool DoubleNavigator::operator==(const DoubleNavigatorPtr& other) const
    {
        if (!other)
        {
            return false;
        }
        return *this == *other;
    }

    /* static methods */
    DoubleNavigatorPtr DoubleNavigator::FromAronDoublePtr(const data::AronDoublePtr& aron)
    {
        return std::make_shared<DoubleNavigator>(aron);
    }

    data::AronDoublePtr DoubleNavigator::ToAronDoublePtr(const DoubleNavigatorPtr& navigator)
    {
        return navigator ? navigator->toAronDoublePtr() : nullptr;
    }


    /* public member functions */
    data::AronDoublePtr DoubleNavigator::toAronDoublePtr() const
    {
        return aron;
    }

    /* virtual implementations */
    std::string DoubleNavigator::getName() const
    {
        return "data::AronDouble";
    }

    bool DoubleNavigator::fullfillsType(const typenavigator::NavigatorPtr& type) const
    {
        const auto f = FloatNavigator();
        return type->getDescriptor() == type::Descriptor::eDouble || f.fullfillsType(type);
    }

    typenavigator::NavigatorPtr DoubleNavigator::recalculateType() const
    {
        return std::make_shared<typenavigator::DoubleNavigator>(getPath());
    }
}
