/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#include "Float.h"

namespace armarx::aron::datanavigator
{
    /* constructors */
    FloatNavigator::FloatNavigator(const data::AronFloatPtr& o, const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::eFloat, path),
        detail::NavigatorBase<data::AronFloat, FloatNavigator>(o)
    {
    }

    FloatNavigator::FloatNavigator(const Path& path) :
        aron::Navigator<data::Descriptor, data::AronData>::Navigator(data::Descriptor::eFloat, path)
    {
    }

    FloatNavigator::FloatNavigator(const float d, const Path& path) :
        FloatNavigator(data::AronFloatPtr(new data::AronFloat(d)), path)
    {
    }

    /* operators */
    bool FloatNavigator::operator==(const FloatNavigator& other) const
    {
        const auto& otherAron = other.toAronFloatPtr();
        if (this->aron->value != otherAron->value)
        {
            return false;
        }
        return true;
    }
    bool FloatNavigator::operator==(const FloatNavigatorPtr& other) const
    {
        if (!other)
        {
            return false;
        }
        return *this == *other;
    }


    /* static methods */
    FloatNavigatorPtr FloatNavigator::FromAronFloatPtr(const data::AronFloatPtr& aron)
    {
        return std::make_shared<FloatNavigator>(aron);
    }

    data::AronFloatPtr FloatNavigator::ToAronFloatPtr(const FloatNavigatorPtr& navigator)
    {
        return navigator ? navigator->toAronFloatPtr() : nullptr;
    }

    /* public member functions */
    data::AronFloatPtr FloatNavigator::toAronFloatPtr() const
    {
        return aron;
    }


    /* virtual implementations */
    std::string FloatNavigator::getName() const
    {
        return "data::AronFloat";
    }

    bool FloatNavigator::fullfillsType(const typenavigator::NavigatorPtr& type) const
    {
        return type->getDescriptor() == type::Descriptor::eFloat;
    }

    typenavigator::NavigatorPtr FloatNavigator::recalculateType() const
    {
        return std::make_shared<typenavigator::FloatNavigator>(getPath());
    }
}
