/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

#pragma once

// STD/STL
#include <string>
#include <map>

// Base Class
#include "../detail/ContainerNavigatorBase.h"

namespace armarx::aron::typenavigator
{
    class PairNavigator;
    typedef std::shared_ptr<PairNavigator> PairNavigatorPtr;

    class PairNavigator :
            virtual public detail::ContainerNavigatorBase<type::AronPair, PairNavigator>
    {
    public:
        // constructors
        PairNavigator(const Path& path = Path());
        PairNavigator(const type::AronPairPtr&, const Path& path = Path());

        // operators
        virtual bool operator==(const PairNavigator&) const override;

        // public member functions
        std::pair<NavigatorPtr, NavigatorPtr> getAcceptedTypes() const;
        NavigatorPtr getFirstAcceptedType() const;
        NavigatorPtr getSecondAcceptedType() const;
        void addAcceptedType(const NavigatorPtr&);
        void setFirstAcceptedType(const NavigatorPtr&);
        void setSecondAcceptedType(const NavigatorPtr&);

        type::AronPairPtr toAronPairPtr() const;

        // virtual implementations
        std::string getName() const override;
        virtual std::vector<NavigatorPtr> getChildren() const override;
        virtual size_t childrenSize() const override;

    private:
        // members
        NavigatorPtr acceptedType1;
        NavigatorPtr acceptedType2;

    };
}
