/*
 * This file is part of ArmarX.
 *
 * Copyright (C) 2012-2016, High Performance Humanoid Technologies (H2T),
 * Karlsruhe Institute of Technology (KIT), all rights reserved.
 *
 * ArmarX is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * ArmarX is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * @author     Fabian Peller-Konrad (fabian dot peller-konrad at kit dot edu)
 * @copyright  http://www.gnu.org/licenses/gpl-2.0.txt
 *             GNU General Public License
 */

// Header
#include "IntEnum.h"

namespace armarx::aron::typenavigator
{
    // constructors
    IntEnumNavigator::IntEnumNavigator(const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eIntEnum, path)
    {
    }

    IntEnumNavigator::IntEnumNavigator(const type::AronIntEnumPtr& o, const Path& path) :
        aron::Navigator<type::Descriptor, type::AronType>::Navigator(type::Descriptor::eIntEnum, path),
        detail::NavigatorBase<type::AronIntEnum, IntEnumNavigator>(o)
    {
    }

    // operators
    bool IntEnumNavigator::operator==(const IntEnumNavigator& other) const
    {
        if (getMaybe() != other.getMaybe())
        {
            return false;
        }
        return getEnumName() == other.getEnumName();
    }

    std::map<std::string, int> IntEnumNavigator::getAcceptedValueMap() const
    {
        return this->aron->acceptedValues;
    }

    std::vector<std::string> IntEnumNavigator::getAcceptedValueNames() const
    {
        std::vector<std::string> names;
        for (const auto& [k, _] : aron->acceptedValues)
        {
            names.push_back(k);
        }
        return names;
    }

    std::vector<int> IntEnumNavigator::getAcceptedValues() const
    {
        std::vector<int> vals;
        for (const auto& [_, i] : aron->acceptedValues)
        {
            vals.push_back(i);
        }
        return vals;
    }

    std::string IntEnumNavigator::getValueName(int i) const
    {
        for (const auto& [k, v] : this->aron->acceptedValues)
        {
            if (v == i)
            {
                return k;
            }
        }
        throw error::AronException("IntEnumNavigator", "getValueName", "Enum could not be resolved. Input was: " + std::to_string(i), getPath());
    }

    int IntEnumNavigator::getValue(const std::string& s) const
    {
        return this->aron->acceptedValues[s];
    }

    std::string IntEnumNavigator::getEnumName() const
    {
        return this->aron->enumName;
    }

    void IntEnumNavigator::setEnumName(const std::string& s)
    {
        this->aron->enumName = s;
    }

    void IntEnumNavigator::addAcceptedValue(const std::string& s, int i)
    {
        this->aron->acceptedValues[s] = i;
    }

    type::AronIntEnumPtr IntEnumNavigator::toAronIntEnumPtr() const
    {
        return this->aron;
    }

    // virtual implementations
    std::string IntEnumNavigator::getName() const
    {
        return "AronIntEnumType";
    }
    std::vector<NavigatorPtr> IntEnumNavigator::getChildren() const
    {
        return {};
    }
    size_t IntEnumNavigator::childrenSize() const
    {
        return 0;
    }
}

